@@ -1,5 +1,5 @@
 /*
- * $Id: daemon_util.c 6482 2012-07-16 06:46:13Z jerry $
+ * $Id: daemon_util.c 7385 2013-11-05 16:00:17Z sara $
  *
  * Copyright (c) 2008-2009 Nominet UK. All rights reserved.
  *
@@ -74,6 +74,7 @@
 #include "ksm/string_util.h"
 #include "ksm/string_util2.h"
 
+
 /**
  * Use _r() functions on platforms that have. They are thread safe versions of
  * the normal syslog functions. Platforms without _r() usually have thread safe
@@ -200,10 +201,10 @@ getPermsForDrop(DAEMONCONFIG* config)
         temp_char = (char*) xmlXPathCastToString(xpathObj);
         StrAppend(&config->groupname, temp_char);
         StrFree(temp_char);
-        xmlXPathFreeObject(xpathObj);
     } else {
         config->groupname = NULL;
     }
+	xmlXPathFreeObject(xpathObj);
 
     /* Set the user to drop to if specified */
     xpathObj = xmlXPathEvalExpression(user_expr, xpathCtx);
@@ -217,10 +218,10 @@ getPermsForDrop(DAEMONCONFIG* config)
         temp_char = (char*) xmlXPathCastToString(xpathObj);
         StrAppend(&config->username, temp_char);
         StrFree(temp_char);
-        xmlXPathFreeObject(xpathObj);
     } else {
         config->username = NULL;
     }
+	xmlXPathFreeObject(xpathObj);
 
     /* Set uid and gid if required */
     if (config->username != NULL) {
@@ -252,7 +253,6 @@ getPermsForDrop(DAEMONCONFIG* config)
         endgrent();
     }
 
-    xmlXPathFreeContext(xpathCtx);
     xmlRelaxNGFree(schema);
     xmlRelaxNGFreeValidCtxt(rngctx);
     xmlRelaxNGFreeParserCtxt(rngpctx);
@@ -292,13 +292,14 @@ void log_switch(int facility, const char *facility_name, const char *program_nam
 }
 
 
-    void
+void
 log_msg(DAEMONCONFIG *config, int priority, const char *format, ...)
 {
     /* If the variable arg list is bad then random errors can occur */ 
     va_list args;
     if (config && config->debug) priority = LOG_ERR;
     va_start(args, format);
+
 #ifdef HAVE_VSYSLOG_R
     vsyslog_r(priority, &sdata, format, args);
 #else
@@ -392,14 +393,15 @@ usage(const char* prog)
     fprintf(stderr, "Usage: %s [OPTION]...\n", prog);
     fprintf(stderr, "OpenDNSSEC Enforcer version %s\n\n", VERSION);
     fprintf(stderr, "Supported options:\n");
-    fprintf(stderr, "  -c <file>   Use alternate conf.xml.\n");
-    fprintf(stderr, "  -d          Debug.\n");
-    fprintf(stderr, "  -1          Run once, then exit.\n");
+    fprintf(stderr, "  -c <file>    Use alternate conf.xml.\n");
+    fprintf(stderr, "  -d           Debug.\n");
+    fprintf(stderr, "  -1           Run once, then exit.\n");
+    fprintf(stderr, "  -p <policy>  Run once processing only the specified policy, then exit.\n");
 /*    fprintf(stderr, "  -u user     Change effective uid to the specified user.\n");*/
-    fprintf(stderr, "  -P pidfile  Specify the PID file to write.\n");
+    fprintf(stderr, "  -P <pidfile> Specify the PID file to write.\n");
 
-    fprintf(stderr, "  -V          Print version.\n");
-    fprintf(stderr, "  -[?|h]      This help.\n");
+    fprintf(stderr, "  -V           Print version.\n");
+    fprintf(stderr, "  -[?|h]       This help.\n");
 }
 
     static void
@@ -483,7 +485,7 @@ writepid (DAEMONCONFIG *config)
     } else {
         if (S_ISREG(stat_ret.st_mode)) {
             /* The file exists already */
-	    if ((oldpid = readpid(config->pidfile)) == -1) {
+            if ((oldpid = readpid(config->pidfile)) == -1) {
                 /* consider stale pidfile */
                 if (errno != ENOENT) {
                     log_msg(config, LOG_ERR, "cannot read pidfile %s: %s",
@@ -586,13 +588,18 @@ int make_directory(DAEMONCONFIG* config, const char* path) {
 
     *slash = 0;
 
-    stat(parent, &stat_ret);
+    if (stat(parent, &stat_ret) != 0) {
+		if (errno != ENOENT) {
+			log_msg(NULL, LOG_ERR, "cannot stat %s: %s\n",
+					parent, strerror(errno));
+			return 1;
+		}
+	}
 
-    if (!S_ISDIR(stat_ret.st_mode)) {
+	if (!S_ISDIR(stat_ret.st_mode)) {
+		make_directory(config, parent);
+	}
 
-        make_directory(config, parent);
-
-    }
 
     StrFree(parent);
 
@@ -622,7 +629,7 @@ cmdlParse(DAEMONCONFIG* config, int *argc, char **argv)
     /*
      * Read the command line
      */
-    while ((c = getopt(*argc, argv, "1c:hdV?u:P:")) != -1) {
+    while ((c = getopt(*argc, argv, "1c:hdV?u:P:p:")) != -1) {
         switch (c) {
             case '1':
                 config->once = true;
@@ -676,6 +683,11 @@ cmdlParse(DAEMONCONFIG* config, int *argc, char **argv)
                     }
                 }   
                 break;
+            case 'p':
+                config->policy = optarg;
+				config->once = true;
+				fprintf(stdout,  "Will only process policy \"%s\", will only run once. Check the logs for results. \n", config->policy);
+                break;
             case 'h':
                 usage(config->program);
                 exit(0);
@@ -728,6 +740,7 @@ ReadConfig(DAEMONCONFIG *config, int verbose)
     char* rngfilename = OPENDNSSEC_SCHEMA_DIR "/conf.rng";
 
     char* temp_char = NULL;
+    char* str = NULL; /* used to split DSSub command */
 
     FILE *file;
 
@@ -880,6 +893,7 @@ ReadConfig(DAEMONCONFIG *config, int verbose)
         if (status > 0) {
             log_msg(config, LOG_ERR, "Error: unable to convert RolloverNotification %s to seconds, error: %i", temp_char, status);
             StrFree(temp_char);
+			xmlXPathFreeObject(xpathObj);
             return status;
         }
         else if (status == -1) {
@@ -890,12 +904,12 @@ ReadConfig(DAEMONCONFIG *config, int verbose)
             log_msg(config, LOG_INFO, "Rollover Notification Interval: %i", config->rolloverNotify);
         }
         StrFree(temp_char);
-        xmlXPathFreeObject(xpathObj);
     }
     else {
         /* Tag RolloverNotification absent, set rolloverNotify to -1 */
         config->rolloverNotify = -1;
     }
+	xmlXPathFreeObject(xpathObj);
 
     /* Evaluate xpath expression for DelegationSignerSubmitCommand */
     xpathObj = xmlXPathEvalExpression(ds_expr, xpathCtx);
@@ -912,16 +926,25 @@ ReadConfig(DAEMONCONFIG *config, int verbose)
         }
         config->DSSubmitCmd = (char *)xmlXPathCastToString(xpathObj);
 
+		/* If the string ends " --cka_id" strip that off and set flag */
+		str = strstr(config->DSSubmitCmd, " --cka_id");
+		if (str) {
+			config->DSSubCKA_ID = 1;
+			*str = 0;
+		} else {
+			config->DSSubCKA_ID = 0;
+		}
+
         if (verbose) {
             log_msg(config, LOG_INFO, "Using command: %s to submit DS records", config->DSSubmitCmd);
         }
-        xmlXPathFreeObject(xpathObj);
     } else {
         if (verbose) {
             log_msg(config, LOG_INFO, "No DS Submit command supplied");
         }
         config->DSSubmitCmd[0] = '\0';
     }
+	xmlXPathFreeObject(xpathObj);
 
     /* Evaluate xpath expression for SQLite file location */
 		
@@ -1052,15 +1075,13 @@ ReadConfig(DAEMONCONFIG *config, int verbose)
     /* Check that we found one or the other database */
     if(db_found == 0) {
         log_msg(config, LOG_ERR, "Error: unable to find complete database connection expression in %s", filename);
-        xmlXPathFreeContext(xpathCtx);
         xmlFreeDoc(doc);
         return(-1);
     }
 
     /* Check that we found the right database type */
     if (db_found != DbFlavour()) {
-        log_msg(config, LOG_ERR, "Error: database in config file %s does not match libksm", filename);
-        xmlXPathFreeContext(xpathCtx);
+        log_msg(config, LOG_ERR, "Error: Config file %s specifies database type %s but system is compiled to use %s", filename, (db_found==1) ? "MySQL" : "sqlite3", (db_found==2) ? "MySQL" : "sqlite3");
         xmlFreeDoc(doc);
         return(-1);
     }
@@ -1082,6 +1103,9 @@ ReadConfig(DAEMONCONFIG *config, int verbose)
         if (status > 0) {
             log_msg(config, LOG_ERR, "Error: unable to set log user: %s, error: %i", logFacilityName, status);
             StrFree(logFacilityName);
+			xmlXPathFreeObject(xpathObj);
+			xmlXPathFreeContext(xpathCtx);
+			xmlFreeDoc(doc);
             return status;
         }
         config->log_user = my_log_user;
@@ -1097,14 +1121,12 @@ ReadConfig(DAEMONCONFIG *config, int verbose)
             log_msg(config, LOG_INFO, "Using default log user: %s", logFacilityName);
         }
     }
-
     xmlXPathFreeObject(xpathObj);
 
     log_switch(my_log_user, logFacilityName, config->program, verbose);
 
     /* Cleanup */
     /* TODO: some other frees are needed */
-    xmlXPathFreeContext(xpathCtx);
     xmlFreeDoc(doc);
     StrFree(logFacilityName);
     StrFree(filename);
