@@ -1,5 +1,5 @@
 /*
- * $Id: enforcer.c 7092 2013-04-10 12:50:25Z sara $
+ * $Id: enforcer.c 7401 2013-11-14 15:46:25Z sion $
  *
  * Copyright (c) 2008-2009 Nominet UK. All rights reserved.
  *
@@ -33,17 +33,18 @@
  * The bit that makes the daemon do something useful
  */
 
+#include "config.h"
+
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
 #include <stdio.h>
 #include <syslog.h>
+#include <sys/stat.h>
 
 #include <libxml/xmlreader.h>
 #include <libxml/xpath.h>
 
-#include "config.h"
-
 #include "daemon.h"
 #include "daemon_util.h"
 #include "enforcer.h"
@@ -59,7 +60,7 @@
 #include "libhsm.h"
 #include "libhsmdns.h"
 
-    int
+int
 server_init(DAEMONCONFIG *config)
 {
     if (config == NULL) {
@@ -78,7 +79,7 @@ server_init(DAEMONCONFIG *config)
 /*
  * Main loop of enforcerd server
  */
-    void
+void
 server_main(DAEMONCONFIG *config)
 {
     DB_RESULT handle;
@@ -116,9 +117,9 @@ server_main(DAEMONCONFIG *config)
 /*    if (config->manualKeyGeneration == 0) {*/
         /* We keep the HSM connection open for the lifetime of the daemon */
         if (config->configfile != NULL) {
-            result = hsm_open(config->configfile, hsm_prompt_pin, NULL);
+            result = hsm_open(config->configfile, hsm_check_pin);
         } else {
-            result = hsm_open(OPENDNSSEC_CONFIG_FILE, hsm_prompt_pin, NULL);
+            result = hsm_open(OPENDNSSEC_CONFIG_FILE, hsm_check_pin);
         }
         if (result) {
             hsm_error_message = hsm_get_error(ctx);
@@ -200,8 +201,19 @@ server_main(DAEMONCONFIG *config)
         log_msg(config, LOG_INFO, "Connecting to Database...");
         kaspConnect(config, &dbhandle);
 
-        /* Read all policies */
-        status = KsmPolicyInit(&handle, NULL);
+		/* check if any specific policy was passed as an arg */
+		if (config->policy != NULL) {
+			log_msg(config, LOG_INFO, "Will only process policy \"%s\" as specified on the command line with the --policy option.", config->policy);			
+			status = KsmPolicyExists(config->policy);
+			if (status != 0) {
+				log_msg(config, LOG_ERR, "Policy \"%s\" not found. Exiting.", config->policy);
+				unlink(config->pidfile);
+                exit(1);
+			}				
+		}
+        /* Read all policies.
+ 			If config->policy is NULL this will return all the policies, if not NULL then just that policy */
+        status = KsmPolicyInit(&handle, config->policy);
         if (status == 0) {
             /* get the first policy */
             status = KsmPolicy(handle, policy);
@@ -247,8 +259,11 @@ server_main(DAEMONCONFIG *config)
 
         /* Communicate zones to the signer */
         KsmParameterCollectionCache(1); /* Enable caching of policy parameters while in do_communication() */
-        do_communication(config, policy);
-        KsmParameterCollectionCache(0);
+		/* If config->policy is NULL then we were not passed a policy on the cmd line and all the policies 
+		   should be processed. However if we have a specific policy, then the 'policy' parameter will be 
+		   already set to that when we call do_communiciation and only that policy will be processed. */
+        do_communication(config, policy, (config->policy == NULL));
+		KsmParameterCollectionCache(0);
         
         DbFreeResult(handle);
 
@@ -303,7 +318,6 @@ server_main(DAEMONCONFIG *config)
 		/* Make sure that we can still talk to the HSM; this call exits if
 		   we can not (after trying to reconnect) */
 		check_hsm_connection(&ctx, config);
-
     }
 
     /*
@@ -360,7 +374,7 @@ int do_keygen(DAEMONCONFIG *config, KSM_POLICY* policy, hsm_ctx_t *ctx)
 
     /* Check datetime in case it came back NULL */
     if (rightnow == NULL) {
-        log_msg(config, LOG_DEBUG, "Couldn't turn \"now\" into a date, quitting...");
+        log_msg(config, LOG_ERR, "Couldn't turn \"now\" into a date, quitting...");
         exit(1);
     }
 
@@ -384,12 +398,13 @@ int do_keygen(DAEMONCONFIG *config, KSM_POLICY* policy, hsm_ctx_t *ctx)
             log_msg(config, LOG_INFO, "No zones on policy %s, skipping...", policy->name);
             StrFree(rightnow);
             return status; 
-        } 
+		}
     } else {
         log_msg(NULL, LOG_ERR, "Could not count zones on policy %s", policy->name);
         StrFree(rightnow);
         return status; 
     }
+	log_msg(config, LOG_INFO, "%d zone(s) found on policy \"%s\"\n", zone_count, policy->name);
 
     /* Find out how many ksk keys are needed for the POLICY */
     status = KsmKeyPredict(policy->id, KSM_TYPE_KSK, policy->shared_keys, config->interval, &ksks_needed, policy->ksk->rollover_scheme, zone_count);
@@ -403,10 +418,7 @@ int do_keygen(DAEMONCONFIG *config, KSM_POLICY* policy, hsm_ctx_t *ctx)
         log_msg(NULL, LOG_ERR, "Could not count current ksk numbers for policy %s", policy->name);
         /* TODO exit? continue with next policy? */
     }
-    /* Correct for shared keys */
-    if (policy->shared_keys == KSM_KEYS_SHARED) {
-        keys_in_queue /= zone_count;
-    }
+    /* Don't have to adjust the queue for shared keys as the prediction has already taken care of that.*/
 
     new_keys = ksks_needed - keys_in_queue;
     /* fprintf(stderr, "keygen(ksk): new_keys(%d) = keys_needed(%d) - keys_in_queue(%d)\n", new_keys, ksks_needed, keys_in_queue); */
@@ -423,6 +435,12 @@ int do_keygen(DAEMONCONFIG *config, KSM_POLICY* policy, hsm_ctx_t *ctx)
             new_keys = policy->ksk->sm_capacity - current_count;
         }
     }
+	if (new_keys <= 0 ) {
+		log_msg(config, LOG_INFO,"No new KSKs need to be created.\n");
+    }
+    else {
+		log_msg(config, LOG_INFO,"%d new KSK(s) (%d bits) need to be created.\n", new_keys, policy->ksk->bits);
+	}
 
     /* Create the required keys */
     for (i=new_keys ; i > 0 ; i--){
@@ -482,10 +500,7 @@ int do_keygen(DAEMONCONFIG *config, KSM_POLICY* policy, hsm_ctx_t *ctx)
         log_msg(NULL, LOG_ERR, "Could not count current zsk numbers for policy %s", policy->name);
         /* TODO exit? continue with next policy? */
     }
-    /* Correct for shared keys */
-    if (policy->shared_keys == KSM_KEYS_SHARED) {
-        keys_in_queue /= zone_count;
-    }
+    /* Don't have to adjust the queue for shared keys as the prediction has already taken care of that.*/
     /* Might have to account for ksks */
     if (same_keys) {
         keys_in_queue -= ksks_needed;
@@ -507,6 +522,14 @@ int do_keygen(DAEMONCONFIG *config, KSM_POLICY* policy, hsm_ctx_t *ctx)
         }
     }
 
+	if (new_keys <= 0 ) {
+		/* Don't exit here, just fall through to the end */
+		log_msg(config, LOG_INFO, "No new ZSKs need to be created.\n");
+    }
+    else {
+		log_msg(config, LOG_INFO, "%d new ZSK(s) (%d bits) need to be created.\n", new_keys, policy->zsk->bits);
+	}
+
     /* Create the required keys */
     for (i = new_keys ; i > 0 ; i--) {
         if (hsm_supported_algorithm(policy->zsk->algorithm) == 0) {
@@ -560,7 +583,7 @@ int do_keygen(DAEMONCONFIG *config, KSM_POLICY* policy, hsm_ctx_t *ctx)
     return status;
 }
 
-int do_communication(DAEMONCONFIG *config, KSM_POLICY* policy)
+int do_communication(DAEMONCONFIG *config, KSM_POLICY* policy, bool all_policies)
 {
     int status = 0;
     int status2 = 0;
@@ -575,7 +598,7 @@ int do_communication(DAEMONCONFIG *config, KSM_POLICY* policy)
     char* zone_name;
     char* current_policy;
     char* current_filename;
-    char *tag_name;
+    char *tag_name = NULL;
     int zone_id = -1;
     int signer_flag = 1; /* Is the signer responding? (1 == yes) */
     char* ksk_expected = NULL;  /* When is the next ksk rollover expected? */
@@ -687,6 +710,15 @@ int do_communication(DAEMONCONFIG *config, KSM_POLICY* policy)
                 xmlXPathFreeObject(xpathObj);
 
                 if (strcmp(current_policy, policy->name) != 0) {
+					if ( !all_policies ) {
+						/*Only process zones on the policy we have */
+						log_msg(config, LOG_INFO, "Skipping zone %s as not on specified policy \"%s\".", zone_name, policy->name);
+						/* Move onto the next zone*/
+	                    ret = xmlTextReaderRead(reader);
+	                    StrFree(tag_name);
+                    	StrFree(zone_name);							
+						continue;
+					}
 
                     /* Read new Policy */ 
                     kaspSetPolicyDefaults(policy, current_policy);
@@ -727,6 +759,7 @@ int do_communication(DAEMONCONFIG *config, KSM_POLICY* policy)
                 xmlXPathFreeObject(xpathObj);
                 /* TODO should we check that we have not written to this file in this run?*/
                 /* Make sure that enough keys are allocated to this zone */
+
                 status2 = allocateKeysToZone(policy, KSM_TYPE_ZSK, zone_id, config->interval, zone_name, config->manualKeyGeneration, 0);
                 if (status2 != 0) {
                     log_msg(config, LOG_ERR, "Error allocating zsks to zone %s", zone_name);
@@ -749,7 +782,7 @@ int do_communication(DAEMONCONFIG *config, KSM_POLICY* policy)
                 }
 
                 /* turn this zone and policy into a file */
-                status2 = commGenSignConf(zone_name, zone_id, current_filename, policy, &signer_flag, config->interval, config->manualKeyGeneration, config->DSSubmitCmd);
+                status2 = commGenSignConf(zone_name, zone_id, current_filename, policy, &signer_flag, config->interval, config->manualKeyGeneration, config->DSSubmitCmd, config->DSSubCKA_ID);
                 if (status2 == -2) {
                     log_msg(config, LOG_ERR, "Signconf not written for %s", zone_name);
                     /* Don't return? try to parse the rest of the zones? */
@@ -775,7 +808,7 @@ int do_communication(DAEMONCONFIG *config, KSM_POLICY* policy)
 
                     /* Check datetime in case it came back NULL */
                     if (datetime == NULL) {
-                        log_msg(config, LOG_DEBUG, "Couldn't turn \"now\" into a date, quitting...");
+                        log_msg(config, LOG_ERR, "Couldn't turn \"now\" into a date, quiting...");
                         unlink(config->pidfile);
                         exit(1);
                     }
@@ -797,8 +830,8 @@ int do_communication(DAEMONCONFIG *config, KSM_POLICY* policy)
                             if (roll_time <= config->rolloverNotify) {
                                 log_msg(config, LOG_INFO, "Rollover of KSK expected at %s for %s", ksk_expected, zone_name);
                             }
-                            StrFree(ksk_expected);
                         }
+						StrFree(ksk_expected);
                     }
                     StrFree(datetime);
                 }
@@ -830,7 +863,7 @@ int do_communication(DAEMONCONFIG *config, KSM_POLICY* policy)
  *  returns 0 on success and -1 if something went wrong
  *                           -2 if the RequestKeys call failed
  */
-int commGenSignConf(char* zone_name, int zone_id, char* current_filename, KSM_POLICY *policy, int* signer_flag, int run_interval, int man_key_gen, const char* DSSubmitCmd)
+int commGenSignConf(char* zone_name, int zone_id, char* current_filename, KSM_POLICY *policy, int* signer_flag, int run_interval, int man_key_gen, const char* DSSubmitCmd, int DSSubCKA_ID)
 {
     int status = 0;
     int status2 = 0;
@@ -842,15 +875,15 @@ int commGenSignConf(char* zone_name, int zone_id, char* current_filename, KSM_PO
     char *old_filename;     /* Keep a copy of the previous version, just in case! (Also gets
                                round potentially different behaviour of rename over existing
                                file.) */
-    char *signer_command;   /* how we will call the signer */
     int     gencnt;         /* Number of keys in generate state */
+    char *signer_command;   /* how we will call the signer */
     int     NewDS = 0;      /* Did we change the DS Set in any way? */
     char*   datetime = DtParseDateTimeString("now");
 
     /* Check datetime in case it came back NULL */
     if (datetime == NULL) {
         log_msg(NULL, LOG_DEBUG, "Couldn't turn \"now\" into a date, quitting...");
-        exit(1);
+        return -1;
     }
 
     if (zone_name == NULL || current_filename == NULL || policy == NULL)
@@ -874,7 +907,8 @@ int commGenSignConf(char* zone_name, int zone_id, char* current_filename, KSM_PO
     if (file == NULL)
     {
         /* error */
-        log_msg(NULL, LOG_ERR, "Could not open: %s", temp_filename);
+        log_msg(NULL, LOG_ERR, "Could not open: %s (%s)", temp_filename,
+		strerror(errno));
         MemFree(datetime);
         StrFree(temp_filename);
         StrFree(old_filename);
@@ -901,6 +935,9 @@ int commGenSignConf(char* zone_name, int zone_id, char* current_filename, KSM_PO
     if (policy->denial->version == 3)
     {
         fprintf(file, "\t\t\t<NSEC3>\n");
+		if (policy->denial->ttl != 0) {
+			fprintf(file, "\t\t\t\t<TTL>PT%dS</TTL>\n", policy->denial->ttl);
+		}
         if (policy->denial->optout == 1)
         {
             fprintf(file, "\t\t\t\t<OptOut />\n");
@@ -981,12 +1018,6 @@ int commGenSignConf(char* zone_name, int zone_id, char* current_filename, KSM_PO
     fprintf(file, "\t\t\t<Serial>%s</Serial>\n", KsmKeywordSerialValueToName( policy->signer->serial) );
     fprintf(file, "\t\t</SOA>\n");
 
-    if (strncmp(policy->audit, "NULL", 4) != 0) {
-        fprintf(file, "\n");
-        fprintf(file, "\t\t<Audit />\n");
-        fprintf(file, "\n");
-    }
-
     fprintf(file, "\t</Zone>\n");
     fprintf(file, "</SignerConfiguration>\n");
 
@@ -1118,9 +1149,12 @@ int commGenSignConf(char* zone_name, int zone_id, char* current_filename, KSM_PO
             if (status != 0)
             {
                 log_msg(NULL, LOG_ERR, "Could not call signer engine");
-                log_msg(NULL, LOG_INFO, "Will continue: call 'ods-signer update' to manually update zones");
+                log_msg(NULL, LOG_INFO, "Will continue: call '%s' to manually update the zone", signer_command);
                 *signer_flag = 0;
             }
+            else {
+                log_msg(NULL, LOG_INFO, "Called signer engine: %s", signer_command);
+            }
 
             StrFree(signer_command);
         }
@@ -1139,7 +1173,7 @@ int commGenSignConf(char* zone_name, int zone_id, char* current_filename, KSM_PO
     /* If the DS set changed then log/do something about it */
     if (NewDS == 1) {
         log_msg(NULL, LOG_INFO, "DSChanged");
-        status = NewDSSet(zone_id, zone_name, DSSubmitCmd);
+        status = NewDSSet(zone_id, zone_name, DSSubmitCmd, DSSubCKA_ID);
     }
 
     StrFree(old_filename);
@@ -1225,7 +1259,7 @@ int allocateKeysToZone(KSM_POLICY *policy, int key_type, int zone_id, uint16_t i
     /* Check datetime in case it came back NULL */
     if (datetime == NULL) {
         log_msg(NULL, LOG_DEBUG, "Couldn't turn \"now\" into a date, quitting...");
-        exit(1);
+        return -1;
     }
 
     if (policy == NULL) {
@@ -1321,7 +1355,7 @@ int allocateKeysToZone(KSM_POLICY *policy, int key_type, int zone_id, uint16_t i
         } else {
             /* This shouldn't happen */
             log_msg(NULL, LOG_ERR, "KsmKeyGetUnallocated returned bad key_id %d for zone: %s; exiting...", key_pair_id, zone_name);
-            exit(1);
+            return -1;
         }
 
     }
@@ -1440,7 +1474,6 @@ int do_purge(int interval, int policy_id)
     DB_ROW      row = NULL;     /* Row data */
 
     char            buffer[KSM_SQL_SIZE];    /* Long enough for any statement */
-    unsigned int    nchar;          /* Number of characters converted */
 
     int         temp_id = -1;       /* place to store the key id returned */
     char*       temp_loc = NULL;    /* place to store location returned */
@@ -1457,7 +1490,7 @@ int do_purge(int interval, int policy_id)
 
     /* Check datetime in case it came back NULL */
     if (rightnow == NULL) {
-        log_msg(NULL, LOG_DEBUG, "Couldn't turn \"now\" into a date, quitting...");
+        log_msg(NULL, LOG_ERR, "Couldn't turn \"now\" into a date, quitting...");
         exit(1);
     }
 
@@ -1485,23 +1518,20 @@ int do_purge(int interval, int policy_id)
             DdsConditionInt(&sql1, "keypair_id", DQS_COMPARE_EQ, temp_id, 0);
             DdsConditionInt(&sql1, "(state", DQS_COMPARE_NE, KSM_STATE_DEAD, 1);
 
-#ifdef USE_MYSQL
-            nchar = snprintf(buffer, sizeof(buffer),
-                    " or state = %d and DEAD > DATE_ADD('%s', INTERVAL -%d SECOND)) ", KSM_STATE_DEAD, rightnow, interval);
-#else
-            nchar = snprintf(buffer, sizeof(buffer),
-                    " or state = %d and DEAD > DATETIME('%s', '-%d SECONDS')) ", KSM_STATE_DEAD, rightnow, interval);
-#endif /* USE_MYSQL */
-
-			if (nchar >= sizeof(buffer)) {
-				log_msg(NULL, LOG_ERR, "Error: failed to create SQL statement to purge keys\n");
-				DbStringFree(temp_loc);
+			status = DbDateDiff(rightnow, interval, -1, buffer, KSM_SQL_SIZE);
+			if (status != 0) {
+				log_msg(NULL, LOG_ERR, "DbDateDiff failed\n");
+                DbStringFree(temp_loc);
                 DbFreeRow(row);
                 StrFree(rightnow);
-				return(-1);
-			}
+				DusFree(sql);
+				DqsFree(sql1);
+                return status;
+			}	
 
+            StrAppend(&sql1, " or state = 6 and DEAD > ");
             StrAppend(&sql1, buffer);
+            StrAppend(&sql1, ")");
             DqsEnd(&sql1);
 
             status = DbIntQuery(DbHandle(), &count, sql1);
@@ -1512,6 +1542,7 @@ int do_purge(int interval, int policy_id)
                 DbStringFree(temp_loc);
                 DbFreeRow(row);
                 StrFree(rightnow);
+				DusFree(sql);
                 return status;
             }
 
@@ -1521,7 +1552,7 @@ int do_purge(int interval, int policy_id)
                 /* Delete from dnsseckeys */
                 sql2 = DdsInit("dnsseckeys");
                 DdsConditionInt(&sql2, "keypair_id", DQS_COMPARE_EQ, temp_id, 0);
-                DdsEnd(&sql);
+                DdsEnd(&sql2);
 
                 status = DbExecuteSqlNoResult(DbHandle(), sql2);
                 DdsFree(sql2);
@@ -1531,6 +1562,7 @@ int do_purge(int interval, int policy_id)
                     DbStringFree(temp_loc);
                     DbFreeRow(row);
                     StrFree(rightnow);
+					DusFree(sql);
                     return status;
                 }
 
@@ -1547,6 +1579,7 @@ int do_purge(int interval, int policy_id)
                     DbStringFree(temp_loc);
                     DbFreeRow(row);
                     StrFree(rightnow);
+					DusFree(sql);
                     return status;
                 }
 
@@ -1558,6 +1591,7 @@ int do_purge(int interval, int policy_id)
                     DbStringFree(temp_loc);
                     DbFreeRow(row);
                     StrFree(rightnow);
+					DusFree(sql);
                     return -1;
                 }
 
@@ -1566,12 +1600,13 @@ int do_purge(int interval, int policy_id)
                 hsm_key_free(key);
 
                 if (!status) {
-                    log_msg(NULL, LOG_INFO, "Key remove successful.\n");
+                    log_msg(NULL, LOG_INFO, "Key remove successful: %s\n", temp_loc);
                 } else {
-                    log_msg(NULL, LOG_ERR, "Key remove failed.\n");
+                    log_msg(NULL, LOG_ERR, "Key remove failed: %s\n", temp_loc);
                     DbStringFree(temp_loc);
                     DbFreeRow(row);
                     StrFree(rightnow);
+					DusFree(sql);
                     return -1;
                 }
             }
@@ -1598,7 +1633,7 @@ int do_purge(int interval, int policy_id)
     return status;
 }
 
-int NewDSSet(int zone_id, const char* zone_name, const char* DSSubmitCmd) {
+int NewDSSet(int zone_id, const char* zone_name, const char* DSSubmitCmd, int DSSubCKA_ID) {
     int     where = 0;		/* for the SELECT statement */
     char*   sql = NULL;     /* SQL statement (when verifying) */
     char*   sql2 = NULL;    /* SQL statement (if getting DS) */
@@ -1637,6 +1672,8 @@ int NewDSSet(int zone_id, const char* zone_name, const char* DSSubmitCmd) {
     FILE *fp;
     int bytes_written = -1;
 
+	struct stat stat_ret; /* we will test the DSSubmitCmd */
+
     nchar = snprintf(buffer, sizeof(buffer), "(%d, %d, %d, %d, %d, %d, %d, %d)",
             KSM_STATE_PUBLISH, KSM_STATE_READY, KSM_STATE_ACTIVE,
             KSM_STATE_DSSUB, KSM_STATE_DSPUBLISH, KSM_STATE_DSREADY, 
@@ -1839,6 +1876,14 @@ int NewDSSet(int zone_id, const char* zone_name, const char* DSSubmitCmd) {
                 }
             }
             StrAppend(&ds_buffer, temp_char);
+
+			/* Add the CKA_ID if asked */
+			if (DSSubCKA_ID) {
+				StrAppend(&ds_buffer, "; {cka_id = ");
+				StrAppend(&ds_buffer, data3.location);
+				StrAppend(&ds_buffer, "}");
+			}
+
             StrFree(temp_char);
 
 /*            StrAppend(&ds_buffer, "\n;KSK DS record (SHA1):\n");
@@ -1867,22 +1912,37 @@ int NewDSSet(int zone_id, const char* zone_name, const char* DSSubmitCmd) {
     }
 
     if (DSSubmitCmd[0] != '\0') {
-        /* send records to the configured command */
-        fp = popen(DSSubmitCmd, "w");
-        if (fp == NULL) {
-            log_msg(NULL, LOG_ERR, "Failed to run command: %s: %s", DSSubmitCmd, strerror(errno));
-            return -1;
-        }
-        bytes_written = fprintf(fp, "%s", ds_buffer);
-        if (bytes_written < 0) {
-            log_msg(NULL, LOG_ERR, "Failed to write to %s: %s", DSSubmitCmd, strerror(errno));
-            return -1;
-        }
+		/* First check that the command exists */
+		if (stat(DSSubmitCmd, &stat_ret) != 0) {
+			log_msg(NULL, LOG_WARNING, "Cannot stat file %s: %s", DSSubmitCmd, strerror(errno));
+		}
+		/* Then see if it is a regular file, then if usr, grp or all have execute set */
+		else if (S_ISREG(stat_ret.st_mode) && !(stat_ret.st_mode & S_IXUSR || stat_ret.st_mode & S_IXGRP || stat_ret.st_mode & S_IXOTH)) {
+			log_msg(NULL, LOG_WARNING, "File %s is not executable", DSSubmitCmd);
+		}
+		else {
 
-        if (pclose(fp) == -1) {
-            log_msg(NULL, LOG_ERR, "Failed to close %s: %s", DSSubmitCmd, strerror(errno));
-            return -1;
-        }
+			/* send records to the configured command */
+			fp = popen(DSSubmitCmd, "w");
+			if (fp == NULL) {
+				log_msg(NULL, LOG_ERR, "Failed to run command: %s: %s", DSSubmitCmd, strerror(errno));
+				StrFree(insql);
+				return -1;
+			}
+			bytes_written = fprintf(fp, "%s", ds_buffer);
+			if (bytes_written < 0) {
+				log_msg(NULL, LOG_ERR, "Failed to write to %s: %s", DSSubmitCmd, strerror(errno));
+				return -1;
+			}
+
+			if (pclose(fp) == -1) {
+				log_msg(NULL, LOG_ERR, "Failed to close %s: %s", DSSubmitCmd, strerror(errno));
+				StrFree(ds_buffer);
+				StrFree(ds_seen_buffer);
+				StrFree(insql);
+				return -1;
+			}
+		}
     }
 
     StrFree(ds_buffer);
@@ -1913,9 +1973,9 @@ void check_hsm_connection(hsm_ctx_t **ctx, DAEMONCONFIG *config)
 		result = hsm_close();
 
 		if (config->configfile != NULL) {
-			result = hsm_open(config->configfile, hsm_prompt_pin, NULL);
+			result = hsm_open(config->configfile, hsm_check_pin);
 		} else {
-			result = hsm_open(OPENDNSSEC_CONFIG_FILE, hsm_prompt_pin, NULL);
+			result = hsm_open(OPENDNSSEC_CONFIG_FILE, hsm_check_pin);
 		}
 		if (result) {
 			hsm_error_message = hsm_get_error(*ctx);
@@ -1956,4 +2016,3 @@ void check_hsm_connection(hsm_ctx_t **ctx, DAEMONCONFIG *config)
 	}
 
 }
-
