@@ -1,5 +1,5 @@
 /*
- * $Id: ksm_key.c 6352 2012-05-29 08:45:11Z sion $
+ * $Id: ksm_key.c 7221 2013-08-19 14:37:53Z sara $
  *
  * Copyright (c) 2008-2009 Nominet UK. All rights reserved.
  *
@@ -790,7 +790,8 @@ int KsmKeyCountStillGood(int policy_id, int sm, int bits, int algorithm, int int
 
     /* Create the SQL command to interrogate the database */ 
  
-    sql = DqsCountInit("KEYDATA_VIEW");
+     /* Use 'distinct location' here so we don't count multiple entries for zones which share keys*/
+    sql = StrStrdup("SELECT COUNT(DISTINCT location) FROM KEYDATA_VIEW");
     if (policy_id != -1) {
         DqsConditionInt(&sql, "policy_id", DQS_COMPARE_EQ, policy_id, where++);
     }
@@ -985,6 +986,7 @@ int KsmMarkKeysAsDead(int zone_id)
         KsmKeyEnd(result);
     }
 
+	DqsFree(sql);
     return 0;
 }
 
@@ -1022,7 +1024,7 @@ int KsmKillKey(int keypair_id, int zone_id)
     DusSetInt(&sql, "STATE", KSM_STATE_DEAD, set++);
     DusSetString(&sql, "DEAD", now, set++);
     DusConditionInt(&sql, "KEYPAIR_ID", DQS_COMPARE_EQ, keypair_id, 0);
-	if (zone_id != -1) {
+    if (zone_id != -1) {
         DqsConditionInt(&sql, "zone_id", DQS_COMPARE_EQ, zone_id, 1);
     }
     DusEnd(&sql);
