@@ -1,5 +1,5 @@
 /*
- * $Id: ksm_parameter.c 3059 2010-03-16 11:52:08Z sion $
+ * $Id: ksm_parameter.c 6379 2012-06-05 08:52:37Z jerry $
  *
  * Copyright (c) 2008-2009 Nominet UK. All rights reserved.
  *
@@ -382,6 +382,21 @@ int KsmCollectionInit(KSM_PARCOLL* data)
  *                  output.
 -*/
 
+static KSM_PARCOLL __parcoll_cache;
+static int __parcoll_cache_policy_id;
+static int __parcoll_cached = 0;
+static int __parcoll_cache_enabled = 0;
+
+void KsmParameterCollectionCache(int enable) {
+    if (enable && !__parcoll_cache_enabled) {
+        __parcoll_cache_enabled = 1;
+        __parcoll_cached = 0;
+    }
+    else if (!enable && __parcoll_cache_enabled) {
+        __parcoll_cache_enabled = 0;
+    }
+}
+
 int KsmParameterCollection(KSM_PARCOLL* data, int policy_id)
 {
     int status = 0;
@@ -392,6 +407,11 @@ int KsmParameterCollection(KSM_PARCOLL* data, int policy_id)
         return MsgLog(KSM_INVARG, "NULL data");
     }
 
+    if (__parcoll_cache_enabled && __parcoll_cached && __parcoll_cache_policy_id == policy_id) {
+        memcpy(data, &__parcoll_cache, sizeof(KSM_PARCOLL));
+        return 0;
+    }
+
     status = KsmParameterValue(KSM_PAR_CLOCKSKEW_STRING, KSM_PAR_CLOCKSKEW_CAT, &(data->clockskew), policy_id, &param_id);
     if (status > 0) return status;
 
@@ -458,6 +478,12 @@ int KsmParameterCollection(KSM_PARCOLL* data, int policy_id)
         data->kskroll = KSM_ROLL_DEFAULT;
     /*}*/
 
+    if (__parcoll_cache_enabled) {
+        memcpy(&__parcoll_cache, data, sizeof(KSM_PARCOLL));
+        __parcoll_cache_policy_id = policy_id;
+        __parcoll_cached = 1;
+    }
+
     return 0;
 }
 
