@@ -1,5 +1,5 @@
 /*
- * $Id: adfile.h 4566 2011-03-15 14:02:32Z matthijs $
+ * $Id: adfile.h 6215 2012-03-20 13:30:51Z matthijs $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -40,46 +40,27 @@
 
 #include <stdio.h>
 
-struct zone_struct;
-
 /**
  * File adapter.
  *
  */
 /** NULL */
 
-/**
- * Initialize file adapters.
- * \return ods_status status
- *
- */
-ods_status adfile_init(void);
-
 /**
  * Read zone from input file adapter.
- * \param[in] zone zone structure
- * \param[in] filename read from this specific file
+ * \param[in] zone zone reference
  * \return ods_status status
  *
  */
-ods_status adfile_read(struct zone_struct* zone, const char* filename);
-
-/**
- * Read zone from backup file.
- * \param[in] zone zone structure
- * \param[in] filename read from this specific file
- * \return ods_status status
- *
- */
-ods_status adbackup_read(struct zone_struct* zone, const char* filename);
+ods_status adfile_read(void* zone);
 
 /**
  * Write zone to output file adapter.
- * \param[in] zone zone structure
+ * \param[in] zone zone reference
  * \param[in] filename write to this specific file
  * \return ods_status status
  *
  */
-ods_status adfile_write(struct zone_struct* zone, const char* filename);
+ods_status adfile_write(void* zone, const char* filename);
 
 #endif /* ADAPTER_ADFILE_H */
