@@ -1,5 +1,5 @@
 /*
- * $Id: signconfparser.c 5809 2011-10-25 11:12:50Z matthijs $
+ * $Id: signconfparser.c 7354 2013-10-09 12:36:03Z matthijs $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -27,16 +27,14 @@
  */
 
 /**
- *
  * Parsing signer configuration files.
+ *
  */
 
 #include "parser/confparser.h"
 #include "parser/signconfparser.h"
-#include "shared/allocator.h"
 #include "shared/duration.h"
 #include "shared/log.h"
-#include "signer/keys.h"
 
 #include <libxml/parser.h>
 #include <libxml/xpath.h>
@@ -52,7 +50,7 @@ static const char* parser_str = "parser";
  *
  */
 keylist_type*
-parse_sc_keys(allocator_type* allocator, const char* cfgfile)
+parse_sc_keys(void* sc, const char* cfgfile)
 {
     xmlDocPtr doc = NULL;
     xmlXPathContextPtr xpathCtx = NULL;
@@ -66,26 +64,22 @@ parse_sc_keys(allocator_type* allocator, const char* cfgfile)
     char* algorithm = NULL;
     int ksk, zsk, publish, i;
 
-    if (!cfgfile) {
-        ods_log_error("[%s] could not parse <Keys>, no cfgfile given",
-            parser_str);
+    if (!cfgfile || !sc) {
         return NULL;
     }
-    ods_log_assert(cfgfile);
-
     /* Load XML document */
     doc = xmlParseFile(cfgfile);
     if (doc == NULL) {
-        ods_log_error("[%s] could not parse <Keys>, xmlParseFile failed",
-            parser_str);
+        ods_log_error("[%s] unable to parse <Keys>: "
+            "xmlParseFile() failed", parser_str);
         return NULL;
     }
     /* Create xpath evaluation context */
     xpathCtx = xmlXPathNewContext(doc);
     if(xpathCtx == NULL) {
         xmlFreeDoc(doc);
-        ods_log_error("[%s] could not parse <Keys>, xmlXPathNewContext failed",
-            parser_str);
+        ods_log_error("[%s] unable to parse <Keys>: "
+            "xmlXPathNewContext() failed", parser_str);
         return NULL;
     }
     /* Evaluate xpath expression */
@@ -94,12 +88,13 @@ parse_sc_keys(allocator_type* allocator, const char* cfgfile)
     if(xpathObj == NULL) {
         xmlXPathFreeContext(xpathCtx);
         xmlFreeDoc(doc);
-        ods_log_error("[%s] could not parse <Keys>, xmlXPathEvalExpression "
-            "failed", parser_str);
+        ods_log_error("[%s] unable to parse <Keys>: "
+            "xmlXPathEvalExpression() failed", parser_str);
         return NULL;
     }
-
-    kl = keylist_create(allocator);
+    /* Parse keys */
+    kl = keylist_create(sc);
+    ods_log_assert(kl);
     if (xpathObj->nodesetval && xpathObj->nodesetval->nodeNr > 0) {
         for (i = 0; i < xpathObj->nodesetval->nodeNr; i++) {
             locator = NULL;
@@ -128,7 +123,7 @@ parse_sc_keys(allocator_type* allocator, const char* cfgfile)
             }
             if (locator && algorithm && flags) {
                 /* search for duplicates */
-                new_key = keylist_lookup(kl, locator);
+                new_key = keylist_lookup_by_locator(kl, locator);
                 if (new_key &&
                     new_key->algorithm == (uint8_t) atoi(algorithm) &&
                     new_key->flags == (uint32_t) atoi(flags) &&
@@ -136,27 +131,23 @@ parse_sc_keys(allocator_type* allocator, const char* cfgfile)
                     new_key->ksk == ksk &&
                     new_key->zsk == zsk) {
                     /* duplicate */
-                    ods_log_warning("[%s] found duplicate key %s, skipping",
-                        parser_str, locator);
+                    ods_log_warning("[%s] unable to push duplicate key %s "
+                        "to keylist, skipping", parser_str, locator);
                 } else {
-                    new_key = key_create(allocator, locator,
+                    (void) keylist_push(kl, locator,
                         (uint8_t) atoi(algorithm), (uint32_t) atoi(flags),
                         publish, ksk, zsk);
-                    if (keylist_push(kl, new_key) != ODS_STATUS_OK) {
-                        ods_log_error("[%s] failed to push key %s to keylist",
-                            parser_str, locator);
-                    }
                 }
             } else {
-                ods_log_error("[%s] Key missing required elements, skipping",
+                ods_log_error("[%s] unable to push key to keylist: <Key> "
+                    "is missing required elements, skipping",
                     parser_str);
             }
-            free((void*)locator);
+            /* free((void*)locator); */
             free((void*)algorithm);
             free((void*)flags);
         }
     }
-
     xmlXPathFreeObject(xpathObj);
     xmlXPathFreeContext(xpathCtx);
     if (doc) {
@@ -282,6 +273,22 @@ parse_sc_dnskey_ttl(const char* cfgfile)
 }
 
 
+duration_type*
+parse_sc_nsec3param_ttl(const char* cfgfile)
+{
+    duration_type* duration = NULL;
+    const char* str = parse_conf_string(cfgfile,
+        "//SignerConfiguration/Zone/Denial/NSEC3/TTL",
+        0);
+    if (!str) {
+        return NULL;
+    }
+    duration = duration_create_from_string(str);
+    free((void*)str);
+    return duration;
+}
+
+
 duration_type*
 parse_sc_soa_ttl(const char* cfgfile)
 {
@@ -328,7 +335,6 @@ parse_sc_nsec_type(const char* cfgfile)
         free((void*)str);
         return LDNS_RR_TYPE_NSEC3;
     }
-
     str = parse_conf_string(cfgfile,
         "//SignerConfiguration/Zone/Denial/NSEC",
         0);
@@ -336,7 +342,6 @@ parse_sc_nsec_type(const char* cfgfile)
         free((void*)str);
         return LDNS_RR_TYPE_NSEC;
     }
-
     return LDNS_RR_TYPE_FIRST;
 }
 
@@ -394,21 +399,6 @@ parse_sc_nsec3_optout(const char* cfgfile)
 }
 
 
-int
-parse_sc_audit(const char* cfgfile)
-{
-    int ret = 0;
-    const char* str = parse_conf_string(cfgfile,
-        "//SignerConfiguration/Zone/Audit",
-        0);
-    if (str) {
-        ret = 1;
-        free((void*)str);
-    }
-    return ret;
-}
-
-
 /**
  * Parse elements from the configuration file.
  *
