@@ -1,5 +1,5 @@
 /*
- * $Id: zonelistparser.c 5817 2011-10-26 14:35:36Z matthijs $
+ * $Id: zonelistparser.c 7041 2013-02-15 09:09:02Z matthijs $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -27,17 +27,17 @@
  */
 
 /**
- *
  * Parsing zonelist files.
+ *
  */
 
 #include "adapter/adapter.h"
 #include "parser/zonelistparser.h"
 #include "shared/file.h"
 #include "shared/log.h"
+#include "shared/status.h"
 #include "signer/zonelist.h"
 #include "signer/zone.h"
-#include "shared/status.h"
 
 #include <libxml/xpath.h>
 #include <libxml/xmlreader.h>
@@ -77,18 +77,16 @@ parse_zonelist_element(xmlXPathContextPtr xpathCtx, xmlChar* expr)
  *
  */
 static adapter_type*
-zlp_adapter(xmlNode* curNode, adapter_mode type, int inbound)
+zlp_adapter(xmlNode* curNode, adapter_mode type, unsigned inbound)
 {
     const char* file = NULL;
     adapter_type* adapter = NULL;
-
     file = (const char*) xmlNodeGetContent(curNode);
     if (!file) {
         ods_log_error("[%s] unable to read %s adapter", parser_str,
             inbound?"input":"output");
         return NULL;
     }
-
     adapter = adapter_create(file, type, inbound);
     free((void*)file);
     return adapter;
@@ -105,26 +103,38 @@ parse_zonelist_adapter(xmlXPathContextPtr xpathCtx, xmlChar* expr,
 {
     xmlXPathObjectPtr xpathObj = NULL;
     xmlNode* curNode = NULL;
+    xmlChar* type = NULL;
     adapter_type* adapter = NULL;
     int i = 0;
 
     if (!xpathCtx || !expr) {
         return NULL;
     }
-
     xpathObj = xmlXPathEvalExpression(expr, xpathCtx);
     if (xpathObj == NULL) {
-        ods_log_error("[%s] unable to evaluate xpath expression %s",
-            parser_str, expr);
+        ods_log_error("[%s] unable to parse adapter: xmlPathEvalExpression() "
+            "failed (expr %s)", parser_str, expr);
         return NULL;
     }
-
     if (xpathObj->nodesetval) {
         for (i=0; i < xpathObj->nodesetval->nodeNr; i++) {
             curNode = xpathObj->nodesetval->nodeTab[i]->xmlChildrenNode;
             while (curNode) {
                 if (xmlStrEqual(curNode->name, (const xmlChar*)"File")) {
                     adapter = zlp_adapter(curNode, ADAPTER_FILE, inbound);
+                } else if (xmlStrEqual(curNode->name,
+                    (const xmlChar*)"Adapter")) {
+                    type = xmlGetProp(curNode, (const xmlChar*)"type");
+                    if (xmlStrEqual(type, (const xmlChar*)"File")) {
+                        adapter = zlp_adapter(curNode, ADAPTER_FILE, inbound);
+                    } else if (xmlStrEqual(type, (const xmlChar*)"DNS")) {
+                        adapter = zlp_adapter(curNode, ADAPTER_DNS, inbound);
+                    } else {
+                        ods_log_error("[%s] unable to parse %s adapter: "
+                            "unknown type", parser_str, (const char*) type);
+                    }
+                    free((void*)type);
+                    type = NULL;
                 }
                 if (adapter) {
                     break;
@@ -151,7 +161,6 @@ parse_zonelist_adapters(xmlXPathContextPtr xpathCtx, zone_type* zone)
     if (!xpathCtx || !zone) {
         return;
     }
-
     zone->adinbound  = parse_zonelist_adapter(xpathCtx, i_expr, 1);
     zone->adoutbound = parse_zonelist_adapter(xpathCtx, o_expr, 0);
     return;
@@ -163,7 +172,7 @@ parse_zonelist_adapters(xmlXPathContextPtr xpathCtx, zone_type* zone)
  *
  */
 ods_status
-parse_zonelist_zones(struct zonelist_struct* zlist, const char* zlfile)
+parse_zonelist_zones(void* zlist, const char* zlfile)
 {
     char* tag_name = NULL;
     char* zone_name = NULL;
@@ -173,31 +182,19 @@ parse_zonelist_zones(struct zonelist_struct* zlist, const char* zlfile)
     xmlTextReaderPtr reader = NULL;
     xmlDocPtr doc = NULL;
     xmlXPathContextPtr xpathCtx = NULL;
-
     xmlChar* name_expr = (unsigned char*) "name";
     xmlChar* policy_expr = (unsigned char*) "//Zone/Policy";
     xmlChar* signconf_expr = (unsigned char*) "//Zone/SignerConfiguration";
 
-    if (!zlist) {
-        ods_log_error("[%s] unable to parse zone list: no storage",
-            parser_str);
+    if (!zlist || !zlfile) {
         return ODS_STATUS_ASSERT_ERR;
     }
-    ods_log_assert(zlist);
-
-    if (!zlfile) {
-        ods_log_error("[%s] unable to parse zone list: no filename",
-            parser_str);
-        return ODS_STATUS_ASSERT_ERR;
-    }
-    ods_log_assert(zlfile);
-
     reader = xmlNewTextReaderFilename(zlfile);
     if (!reader) {
-        ods_log_error("[%s] unable to open file %s", parser_str, zlfile);
+        ods_log_error("[%s] unable to parse zonelist: failed to open file %s",
+            parser_str, zlfile);
         return ODS_STATUS_XML_ERR;
     }
-
     ret = xmlTextReaderRead(reader);
     while (ret == XML_READER_TYPE_ELEMENT) {
         tag_name = (char*) xmlTextReaderLocalName(reader);
@@ -208,8 +205,8 @@ parse_zonelist_zones(struct zonelist_struct* zlist, const char* zlfile)
             zone_name = (char*) xmlTextReaderGetAttribute(reader,
                 name_expr);
             if (!zone_name || strlen(zone_name) <= 0) {
-                ods_log_error("[%s] unable to extract zone name from "
-                    "zonelist", parser_str);
+                ods_log_alert("[%s] unable to extract zone name from "
+                    "zonelist %s, skipping...", parser_str, zlfile);
                 if (zone_name) {
                     free((void*) zone_name);
                 }
@@ -217,7 +214,6 @@ parse_zonelist_zones(struct zonelist_struct* zlist, const char* zlfile)
                 ret = xmlTextReaderRead(reader);
                 continue;
             }
-
             /* Expand this node to get the rest of the info */
             xmlTextReaderExpand(reader);
             doc = xmlTextReaderCurrentDoc(reader);
@@ -225,39 +221,40 @@ parse_zonelist_zones(struct zonelist_struct* zlist, const char* zlfile)
                 xpathCtx = xmlXPathNewContext(doc);
             }
             if (doc == NULL || xpathCtx == NULL) {
-                ods_log_error("[%s] unable to read zone %s; skipping",
+                ods_log_alert("[%s] unable to read zone %s, skipping...",
                    parser_str, zone_name);
                 ret = xmlTextReaderRead(reader);
                 free((void*) zone_name);
                 free((void*) tag_name);
-                if (xpathCtx) {
-                    xmlXPathFreeContext(xpathCtx);
-                    xpathCtx = NULL;
-                }
                 continue;
             }
-
             /* That worked, now read out the contents... */
             new_zone = zone_create(zone_name, LDNS_RR_CLASS_IN);
-            new_zone->policy_name = parse_zonelist_element(xpathCtx,
-                policy_expr);
-            new_zone->signconf_filename = parse_zonelist_element(xpathCtx,
-                signconf_expr);
-            parse_zonelist_adapters(xpathCtx, new_zone);
-            if (!new_zone->policy_name || !new_zone->signconf_filename ||
-                !new_zone->adinbound || !new_zone->adoutbound) {
-                zone_cleanup(new_zone);
-                new_zone = NULL;
+            if (new_zone) {
+                new_zone->policy_name = parse_zonelist_element(xpathCtx,
+                    policy_expr);
+                new_zone->signconf_filename = parse_zonelist_element(xpathCtx,
+                    signconf_expr);
+                parse_zonelist_adapters(xpathCtx, new_zone);
+                if (!new_zone->policy_name || !new_zone->signconf_filename ||
+                    !new_zone->adinbound || !new_zone->adoutbound) {
+                    zone_cleanup(new_zone);
+                    new_zone = NULL;
+                    ods_log_crit("[%s] unable to create zone %s", parser_str,
+                        zone_name);
+                    error = 1;
+                } else if (zonelist_add_zone((zonelist_type*) zlist, new_zone)
+                    == NULL) {
+                    ods_log_crit("[%s] unable to add zone %s", parser_str,
+                        zone_name);
+                    zone_cleanup(new_zone);
+                    new_zone = NULL;
+                    error = 1;
+                }
+            } else {
                 ods_log_crit("[%s] unable to create zone %s", parser_str,
                     zone_name);
                 error = 1;
-            } else if (zonelist_add_zone((zonelist_type*) zlist, new_zone)
-                == NULL) {
-                ods_log_error("[%s] unable to add zone %s", parser_str,
-                    zone_name);
-                zone_cleanup(new_zone);
-                new_zone = NULL;
-                error = 1;
             }
             xmlXPathFreeContext(xpathCtx);
             xpathCtx = NULL;
@@ -280,7 +277,8 @@ parse_zonelist_zones(struct zonelist_struct* zlist, const char* zlfile)
         xmlFreeDoc(doc);
     }
     if (ret != 0) {
-        ods_log_error("[%s] error parsing file %s", parser_str, zlfile);
+        ods_log_error("[%s] unable to parse zonelist: parse error in %s",
+            parser_str, zlfile);
         return ODS_STATUS_PARSE_ERR;
     }
     return ODS_STATUS_OK;
