@@ -32,9 +32,7 @@
  */
 
 #include "config.h"
-#include "daemon/worker.h"
 #include "scheduler/fifoq.h"
-#include "shared/allocator.h"
 #include "shared/log.h"
 
 #include <ldns/ldns.h>
@@ -51,25 +49,19 @@ fifoq_create(allocator_type* allocator)
 {
     fifoq_type* fifoq;
     if (!allocator) {
-        ods_log_error("[%s] unable to create: no allocator available",
+        return NULL;
+    }
+    fifoq = (fifoq_type*) allocator_alloc(allocator, sizeof(fifoq_type));
+    if (!fifoq) {
+        ods_log_error("[%s] unable to create fifoq: allocator_alloc() failed",
             fifoq_str);
         return NULL;
     }
-    ods_log_assert(allocator);
-
-    fifoq = (fifoq_type*) allocator_alloc(allocator, sizeof(fifoq_type));
-    if (!fifoq) {
-        ods_log_error("[%s] unable to create: allocator failed", fifoq_str);
-        return NULL;
-    }
-    ods_log_assert(fifoq);
-
     fifoq->allocator = allocator;
     fifoq_wipe(fifoq);
     lock_basic_init(&fifoq->q_lock);
     lock_basic_set(&fifoq->q_threshold);
     lock_basic_set(&fifoq->q_nonfull);
-    fifoq->q_locked = 0;
     return fifoq;
 }
 
@@ -82,7 +74,6 @@ void
 fifoq_wipe(fifoq_type* q)
 {
     size_t i = 0;
-
     for (i=0; i < FIFOQ_MAX_COUNT; i++) {
         q->blob[i] = NULL;
         q->owner[i] = NULL;
@@ -101,14 +92,9 @@ fifoq_pop(fifoq_type* q, worker_type** worker)
 {
     void* pop = NULL;
     size_t i = 0;
-
-    if (!q) {
+    if (!q || q->count <= 0) {
         return NULL;
     }
-    if (q->count <= 0) {
-        return NULL;
-    }
-
     pop = q->blob[0];
     *worker = q->owner[0];
     for (i = 0; i < q->count-1; i++) {
@@ -116,9 +102,11 @@ fifoq_pop(fifoq_type* q, worker_type** worker)
         q->owner[i] = q->owner[i+1];
     }
     q->count -= 1;
-
     if (q->count <= (size_t) FIFOQ_MAX_COUNT * 0.1) {
-        /* notify waiting workers that they can start queuing again */
+        /**
+         * Notify waiting workers that they can start queuing again
+         * If no workers are waiting, this call has no effect.
+         */
         lock_basic_broadcast(&q->q_nonfull);
     }
     return pop;
@@ -132,12 +120,15 @@ fifoq_pop(fifoq_type* q, worker_type** worker)
 ods_status
 fifoq_push(fifoq_type* q, void* item, worker_type* worker, int* tries)
 {
-    ods_log_assert(q);
-    ods_log_assert(item);
-    ods_log_assert(worker);
-
+    if (!q || !item || !worker) {
+        return ODS_STATUS_ASSERT_ERR;
+    }
     if (q->count >= FIFOQ_MAX_COUNT) {
-        /* #262 if drudgers remain on hold, do additional broadcast */
+        /**
+         * #262:
+         * If drudgers remain on hold, do additional broadcast.
+         * If no drudgers are waiting, this call has no effect.
+         */
         if (*tries > FIFOQ_TRIES_COUNT) {
             lock_basic_broadcast(&q->q_threshold);
             ods_log_debug("[%s] queue full, notify drudgers again", fifoq_str);
@@ -146,14 +137,14 @@ fifoq_push(fifoq_type* q, void* item, worker_type* worker, int* tries)
         }
         return ODS_STATUS_UNCHANGED;
     }
-
     q->blob[q->count] = item;
     q->owner[q->count] = worker;
     q->count += 1;
     if (q->count == 1) {
-        lock_basic_broadcast(&q->q_threshold);
         ods_log_deeebug("[%s] threshold %u reached, notify drudgers",
             fifoq_str, q->count);
+        /* If no drudgers are waiting, this call has no effect. */
+        lock_basic_broadcast(&q->q_threshold);
     }
     return ODS_STATUS_OK;
 }
@@ -170,16 +161,13 @@ fifoq_cleanup(fifoq_type* q)
     lock_basic_type q_lock;
     cond_basic_type q_threshold;
     cond_basic_type q_nonfull;
-
     if (!q) {
         return;
     }
-    ods_log_assert(q);
     allocator = q->allocator;
     q_lock = q->q_lock;
     q_threshold = q->q_threshold;
     q_nonfull = q->q_nonfull;
-
     allocator_deallocate(allocator, (void*) q);
     lock_basic_off(&q_threshold);
     lock_basic_off(&q_nonfull);
