@@ -1,5 +1,5 @@
 /*
- * $Id: task.h 4550 2011-03-11 11:42:01Z matthijs $
+ * $Id: task.h 6181 2012-02-21 14:12:17Z matthijs $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -41,11 +41,10 @@
 
 enum task_id_enum {
     TASK_NONE = 0,
-    TASK_SIGNCONF,
-    TASK_READ,
+    TASK_SIGNCONF, /* ods-signer update */
+    TASK_READ, /* ods-signer sign */
     TASK_NSECIFY,
-    TASK_SIGN,
-    TASK_AUDIT,
+    TASK_SIGN, /* ods-signer flush */
     TASK_WRITE
 };
 typedef enum task_id_enum task_id;
@@ -60,10 +59,9 @@ struct task_struct {
     task_id interrupt;
     task_id halted;
     time_t when;
+    time_t halted_when;
     time_t backoff;
     int flush;
-    const char* who;
-    ldns_rdf* dname;
     void* zone;
 };
 
@@ -71,21 +69,11 @@ struct task_struct {
  * Create a new task.
  * \param[in] what task identifier
  * \param[in] when scheduled time
- * \param[in] who zone name
- * \param[in] zone pointer to zone
+ * \param[in] zone zone reference
  * \return task_type* created task
  *
  */
-task_type* task_create(task_id what, time_t when, const char* who, void* zone);
-
-/**
- * Recover a task from backup.
- * \param[in] filename where the task backup is stored
- * \param[in] zone pointer to zone structure
- * \return task_type* created task
- *
- */
-task_type* task_recover_from_backup(const char* filename, void* zone);
+task_type* task_create(task_id what, time_t when, void* zone);
 
 /**
  * Backup task.
@@ -95,13 +83,6 @@ task_type* task_recover_from_backup(const char* filename, void* zone);
  */
 void task_backup(FILE* fd, task_type* task);
 
-/**
- * Clean up task.
- * \param[in] task task
- *
- */
-void task_cleanup(task_type* task);
-
 /**
  * Compare tasks.
  * \param[in] a one task
@@ -126,14 +107,14 @@ char* task2str(task_type* task, char* buftask);
  * \return const char* string-format of what
  *
  */
-const char* task_what2str(int taskid);
+const char* task_what2str(task_id what);
 
 /**
  * String-format of who.
- * \param[in] what task owner
+ * \param[in] task task
  * \return const char* string-format of who
  */
-const char* task_who2str(const char* who);
+const char* task_who2str(task_type* task);
 
 /**
  * Print task.
@@ -150,4 +131,11 @@ void task_print(FILE* out, task_type* task);
  */
 void task_log(task_type* task);
 
+/**
+ * Clean up task.
+ * \param[in] task task
+ *
+ */
+void task_cleanup(task_type* task);
+
 #endif /* SCHEDULER_TASK_H */
