@@ -36,12 +36,12 @@
 
 #include "config.h"
 #include "shared/allocator.h"
+#include "shared/status.h"
 #include "signer/nsec3params.h"
 #include "signer/rrset.h"
 
 #include <ldns/ldns.h>
-
-struct domain_struct;
+#include <time.h>
 
 /**
  * Denial of Existence data point.
@@ -49,50 +49,60 @@ struct domain_struct;
  */
 typedef struct denial_struct denial_type;
 struct denial_struct {
-    allocator_type* allocator;
-    ldns_rdf* owner;
+    void* zone;
+    void* domain;
+    ldns_rbnode_t* node;
+    ldns_rdf* dname;
     rrset_type* rrset;
-    struct domain_struct* domain;
-    uint8_t bitmap_changed;
-    uint8_t nxt_changed;
+    unsigned bitmap_changed : 1;
+    unsigned nxt_changed : 1;
 };
 
 /**
  * Create new Denial of Existence data point.
- * \param[in] owner owner name of the NSEC or NSEC3 RRset
- * \return denial_type* denial of existence data
+ * \param[in] zoneptr zone reference
+ * \param[in] dname owner name
+ * \return denial_type* denial of existence data point
  *
  */
-denial_type* denial_create(ldns_rdf* owner);
+denial_type* denial_create(void* zoneptr, ldns_rdf* dname);
 
 /**
- * Add NSEC to the Denial of Existence data point.
+ * Apply differences at denial.
+ * \param[in] denial Denial of Existence data point
+ *
+ */
+void denial_diff(denial_type* denial);
+
+/**
+ * Add NSEC(3) to the Denial of Existence data point.
+ * \param[in] denial Denial of Existence data point
+ * \param[in] rr NSEC(3) resource record
+ *
+ */
+void denial_add_rr(denial_type* denial, ldns_rr* rr);
+
+/**
+ * Nsecify Denial of Existence data point.
  * \param[in] denial Denial of Existence data point
  * \param[in] nxt next Denial of Existence data point
- * \param[in] ttl ttl
- * \param[in] klass class
- * \return ods_status status
+ * \param[out] num_added number of RRs added
  *
  */
-ods_status denial_nsecify(denial_type* denial, denial_type* nxt, uint32_t ttl,
-    ldns_rr_class klass);
+void denial_nsecify(denial_type* denial, denial_type* nxt, uint32_t* num_added);
 
 /**
- * Add NSEC3 to the Denial of Existence data point.
- * \param[in] denial Denial of Existence data point
- * \param[in] nxt next Denial of Existence data point
- * \param[in] ttl ttl
- * \param[in] klass class
- * \param[in] nsec3params NSEC3 parameters
- * \return ods_status status
+ * Print Denial of Existence data point.
+ * \param[in] fd file descriptor
+ * \param[in] denial denial of existence data point
+ * \param[out] status status
  *
  */
-ods_status denial_nsecify3(denial_type* denial, denial_type* nxt, uint32_t ttl,
-    ldns_rr_class klass, nsec3params_type* nsec3params);
+void denial_print(FILE* fd, denial_type* denial, ods_status* status);
 
 /**
- * Clean up Denial of Existence data point.
- * \param[in] denial Denial of Existence data point
+ * Cleanup Denial of Existence data point.
+ * \param[in] denial denial of existence data point
  *
  */
 void denial_cleanup(denial_type* denial);
