@@ -1,5 +1,5 @@
 /*
- * $Id: keys.h 5380 2011-08-09 21:00:32Z matthijs $
+ * $Id: keys.h 6215 2012-03-20 13:30:51Z matthijs $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -54,17 +54,15 @@
  */
 typedef struct key_struct key_type;
 struct key_struct {
-    allocator_type* allocator;
-    const char* locator;
     ldns_rr* dnskey;
     hsm_key_t* hsmkey;
     hsm_sign_params_t* params;
+    const char* locator;
     uint8_t algorithm;
     uint32_t flags;
     int publish;
     int ksk;
     int zsk;
-    key_type* next;
 };
 
 /**
@@ -73,51 +71,18 @@ struct key_struct {
  */
 typedef struct keylist_struct keylist_type;
 struct keylist_struct {
-    allocator_type* allocator;
+    void* sc;
+    key_type* keys;
     size_t count;
-    key_type* first_key;
 };
 
-/**
- * Create a new key.
- * \param[in] allocator memory allocator
- * \param[in] locator string that identifies location of key
- * \param[in] algorithm DNSKEY algorithm field value
- * \param[in] flags DNSKEY flags field value
- * \param[in] publish if true, publish key as a DNSKEY
- * \param[in] ksk if true, sign DNSKEY RRset with this key
- * \param[in] zsk if true, sign all but DNSKEY RRset with this key
- * \return key_type* key
- *
- */
-key_type* key_create(allocator_type* allocator, const char* locator,
-    uint8_t algorithm, uint32_t flags, int publish, int ksk, int zsk);
-
-/**
- * Recover key from backup.
- * \param[in] fd file descriptor of key backup file
- * \param[in] allocator memory allocator
- * \return key_type* key
- *
- */
-key_type* key_recover(FILE* fd, allocator_type* allocator);
-
 /**
  * Create a new key list.
- * \param[in] allocator memory allocator
+ * \param[in] sc signer configuration reference
  * \return keylist_type* key list
  *
  */
-keylist_type* keylist_create(allocator_type* allocator);
-
-/**
- * Push a key to the keylist.
- * \param[in] kl key list
- * \param[in] key key
- * \return ods_status status
- *
- */
-ods_status keylist_push(keylist_type* kl, key_type* key);
+keylist_type* keylist_create(void* sc);
 
 /**
  * Lookup a key in the key list by locator.
@@ -126,7 +91,7 @@ ods_status keylist_push(keylist_type* kl, key_type* key);
  * \return key_type* key if it exists, NULL otherwise
  *
  */
-key_type* keylist_lookup(keylist_type* kl, const char* locator);
+key_type* keylist_lookup_by_locator(keylist_type* kl, const char* locator);
 
 /**
  * Lookup a key in the key list by dnskey.
@@ -138,11 +103,19 @@ key_type* keylist_lookup(keylist_type* kl, const char* locator);
 key_type* keylist_lookup_by_dnskey(keylist_type* kl, ldns_rr* dnskey);
 
 /**
- * Clean up key list.
- * \param[in] kl key list to clean up
+ * Push a key to the keylist.
+ * \param[in] kl key list
+ * \param[in] locator string that identifies location of key
+ * \param[in] algorithm DNSKEY algorithm field value
+ * \param[in] flags DNSKEY flags field value
+ * \param[in] publish if true, publish key as a DNSKEY
+ * \param[in] ksk if true, sign DNSKEY RRset with this key
+ * \param[in] zsk if true, sign all but DNSKEY RRset with this key
+ * \return key_type* key
  *
  */
-void keylist_cleanup(keylist_type* kl);
+key_type* keylist_push(keylist_type* kl, const char* locator,
+    uint8_t algorithm, uint32_t flags, int publish, int ksk, int zsk);
 
 /**
  * Print key list.
@@ -152,14 +125,6 @@ void keylist_cleanup(keylist_type* kl);
  */
 void keylist_print(FILE* fd, keylist_type* kl);
 
-/**
- * Backup key list.
- * \param[in] fd file descriptor
- * \param[in] kl key list to print
- *
- */
-void keylist_backup(FILE* fd, keylist_type* kl);
-
 /**
  * Log key list.
  * \param[in] kl key list to print
@@ -168,4 +133,29 @@ void keylist_backup(FILE* fd, keylist_type* kl);
  */
 void keylist_log(keylist_type* kl, const char* name);
 
+/**
+ * Clean up key list.
+ * \param[in] kl key list to clean up
+ *
+ */
+void keylist_cleanup(keylist_type* kl);
+
+/**
+ * Recover key from backup.
+ * \param[in] fd file descriptor of key backup file
+ * \param[in] kl key list to print
+ * \return key_type* key
+ *
+ */
+key_type* key_recover2(FILE* fd, keylist_type* kl);
+
+/**
+ * Backup key list.
+ * \param[in] fd file descriptor
+ * \param[in] kl key list to print
+ * \param[in] version version string
+ *
+ */
+void keylist_backup(FILE* fd, keylist_type* kl, const char* version);
+
 #endif /* SIGNER_KEYS_H */
