@@ -1,5 +1,5 @@
 /*
- * $Id: zonelist.c 7124 2013-05-03 09:49:26Z matthijs $
+ * $Id: zonelist.c 6166 2012-02-14 15:36:44Z matthijs $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -57,17 +57,15 @@ zone_compare(const void* a, const void* b)
 {
     zone_type* x = (zone_type*)a;
     zone_type* y = (zone_type*)b;
-
     ods_log_assert(x);
     ods_log_assert(y);
-
     if (x->klass != y->klass) {
         if (x->klass < y->klass) {
             return -1;
         }
         return 1;
     }
-    return ldns_dname_compare(x->dname, y->dname);
+    return ldns_dname_compare(x->apex, y->apex);
 }
 
 
@@ -78,25 +76,25 @@ zone_compare(const void* a, const void* b)
 zonelist_type*
 zonelist_create(allocator_type* allocator)
 {
-    zonelist_type* zlist;
-    if (!allocator) {
-        ods_log_error("[%s] cannot create: no allocator available", zl_str);
-        return NULL;
+    zonelist_type* zlist = NULL;
+    if (allocator) {
+        zlist = (zonelist_type*) allocator_alloc(allocator, sizeof(zonelist_type));
     }
-    ods_log_assert(allocator);
-
-    zlist = (zonelist_type*) allocator_alloc(allocator, sizeof(zonelist_type));
     if (!zlist) {
-        ods_log_error("[%s] cannot create: allocator failed", zl_str);
+        ods_log_error("[%s] unable to create zonelist: allocator_alloc() "
+            "failed", zl_str);
         return NULL;
     }
-    ods_log_assert(zlist);
-
     zlist->allocator = allocator;
     zlist->zones = ldns_rbtree_create(zone_compare);
+    if (!zlist->zones) {
+        ods_log_error("[%s] unable to create zonelist: ldns_rbtree_create() "
+            "failed", zl_str);
+        allocator_deallocate(allocator, (void*) zlist);
+        return NULL;
+    }
     zlist->last_modified = 0;
     lock_basic_init(&zlist->zl_lock);
-    zlist->zl_locked = 0;
     return zlist;
 }
 
@@ -110,19 +108,14 @@ zonelist_read(zonelist_type* zl, const char* zlfile)
 {
     const char* rngfile = ODS_SE_RNGDIR "/zonelist.rng";
     ods_status status = ODS_STATUS_OK;
-
     ods_log_assert(zlfile);
     ods_log_verbose("[%s] read file %s", zl_str, zlfile);
-
-    /* does the file have no parse errors? */
     status = parse_file_check(zlfile, rngfile);
     if (status != ODS_STATUS_OK) {
-        ods_log_error("[%s] unable to parse file %s: %s", zl_str,
-            zlfile, ods_status2str(status));
+        ods_log_error("[%s] unable to read file: parse error in %s", zl_str,
+            zlfile);
         return status;
     }
-
-    /* ok, parse it */
     return parse_zonelist_zones((struct zonelist_struct*) zl, zlfile);
 }
 
@@ -152,22 +145,11 @@ static zone_type*
 zonelist_lookup_zone(zonelist_type* zonelist, zone_type* zone)
 {
     ldns_rbnode_t* node = LDNS_RBTREE_NULL;
-
-    if (!zonelist || !zonelist->zones) {
-        ods_log_error("[%s] unable to lookup zone: no zonelist", zl_str);
-        return NULL;
-    }
-    ods_log_assert(zonelist);
-    ods_log_assert(zonelist->zones);
-    if (!zone) {
-        ods_log_error("[%s] unable to lookup zone: zone is null", zl_str);
-        return NULL;
-    }
-    ods_log_assert(zone);
-
-    node = ldns_rbtree_search(zonelist->zones, zone);
-    if (node) {
-        return (zone_type*) node->data;
+    if (zonelist && zonelist->zones && zone) {
+        node = ldns_rbtree_search(zonelist->zones, zone);
+        if (node) {
+            return (zone_type*) node->data;
+        }
     }
     return NULL;
 }
@@ -183,22 +165,36 @@ zonelist_lookup_zone_by_name(zonelist_type* zonelist, const char* name,
 {
     zone_type* zone = NULL;
     zone_type* result = NULL;
-
-    if (!zonelist || !zonelist->zones || !name || !klass) {
-        return NULL;
+    if (zonelist && zonelist->zones && name  && klass) {
+        zone = zone_create((char*) name, klass);
+        if (!zone) {
+            ods_log_error("[%s] unable to lookup zone %s: "
+                "zone_create() failed", zl_str, name);
+            /* result stays NULL */
+        } else {
+            result = zonelist_lookup_zone(zonelist, zone);
+            zone_cleanup(zone);
+        }
     }
-    ods_log_assert(zonelist);
-    ods_log_assert(zonelist->zones);
-    ods_log_assert(name);
-    ods_log_assert(klass);
+    return result;
+}
 
-    zone = zone_create((char*) name, klass);
-    if (!zone) {
-        ods_log_error("[%s] unable to lookup zone: create zone failed", zl_str);
-        return NULL;
+
+/**
+ * Lookup zone by dname.
+ *
+ */
+zone_type*
+zonelist_lookup_zone_by_dname(zonelist_type* zonelist, ldns_rdf* dname,
+    ldns_rr_class klass)
+{
+    char* name = NULL;
+    zone_type* result = NULL;
+    if (zonelist && zonelist->zones && dname && klass) {
+        name = ldns_rdf2str(dname);
+        result = zonelist_lookup_zone_by_name(zonelist, name, klass);
+        free((void*)name);
     }
-    result = zonelist_lookup_zone(zonelist, zone);
-    zone_cleanup(zone);
     return result;
 }
 
@@ -211,37 +207,30 @@ zone_type*
 zonelist_add_zone(zonelist_type* zlist, zone_type* zone)
 {
     ldns_rbnode_t* new_node = NULL;
-
     if (!zone) {
-        ods_log_error("[%s] unable to add zone: zone is null", zl_str);
         return NULL;
     }
-    ods_log_assert(zone);
     if (!zlist || !zlist->zones) {
-        ods_log_error("[%s] unable to add zone %s: no zonelist", zl_str,
-            zone->name);
         zone_cleanup(zone);
         return NULL;
     }
-    ods_log_assert(zlist);
-    ods_log_assert(zlist->zones);
-
+    /* look up */
     if (zonelist_lookup_zone(zlist, zone) != NULL) {
         ods_log_warning("[%s] unable to add zone %s: already present", zl_str,
             zone->name);
         zone_cleanup(zone);
         return NULL;
     }
-
+    /* add */
     new_node = zone2node(zone);
     if (ldns_rbtree_insert(zlist->zones, new_node) == NULL) {
-        ods_log_error("[%s] unable to add zone %s: rbtree insert failed",
-            zl_str, zone->name);
+        ods_log_error("[%s] unable to add zone %s: ldns_rbtree_insert() "
+            "failed", zl_str, zone->name);
         free((void*) new_node);
         zone_cleanup(zone);
         return NULL;
     }
-    zone->just_added = 1;
+    zone->zl_status = ZONE_ZL_ADDED;
     zlist->just_added++;
     return zone;
 }
@@ -255,28 +244,23 @@ zone_type*
 zonelist_del_zone(zonelist_type* zlist, zone_type* zone)
 {
     ldns_rbnode_t* old_node = LDNS_RBTREE_NULL;
-
     if (!zone) {
-        ods_log_warning("[%s] unable to delete zone: zone is null", zl_str);
         return NULL;
     }
-    ods_log_assert(zone);
     if (!zlist || !zlist->zones) {
-        ods_log_error("[%s] unable to delete zone %s: no zone list", zl_str,
-            zone->name);
-        return zone;
+        goto zone_not_present;
     }
-    ods_log_assert(zlist);
-    ods_log_assert(zlist->zones);
-
     old_node = ldns_rbtree_delete(zlist->zones, zone);
     if (!old_node) {
-        ods_log_warning("[%s] unable to delete zone %s: not present", zl_str,
-            zone->name);
-        return zone;
+        goto zone_not_present;
     }
     free((void*) old_node);
     return zone;
+
+zone_not_present:
+    ods_log_warning("[%s] unable to delete zone %s: not present", zl_str,
+        zone->name);
+    return zone;
 }
 
 
@@ -308,7 +292,6 @@ zonelist_merge(zonelist_type* zl1, zonelist_type* zl2)
         } else {
             z1 = NULL;
         }
-
         if (!z2) {
             /* no more zones to merge into zl1 */
             return;
@@ -316,7 +299,7 @@ zonelist_merge(zonelist_type* zl1, zonelist_type* zl2)
             /* just add remaining zones from zl2 */
             z2 = zonelist_add_zone(zl1, z2);
             if (!z2) {
-                ods_log_error("[%s] merge failed: z2 not added", zl_str);
+                ods_log_crit("[%s] merge failed: z2 not added", zl_str);
                 return;
             }
             n2 = ldns_rbtree_next(n2);
@@ -325,14 +308,14 @@ zonelist_merge(zonelist_type* zl1, zonelist_type* zl2)
             ret = zone_compare(z1, z2);
             if (ret < 0) {
                 /* remove zone z1, it is not present in the new list zl2 */
-                z1->tobe_removed = 1;
+                z1->zl_status = ZONE_ZL_REMOVED;
                 zl1->just_removed++;
                 n1 = ldns_rbtree_next(n1);
             } else if (ret > 0) {
                 /* add the new zone z2 */
                 z2 = zonelist_add_zone(zl1, z2);
                 if (!z2) {
-                    ods_log_error("[%s] merge failed: z2 not added", zl_str);
+                    ods_log_crit("[%s] merge failed: z2 not added", zl_str);
                     return;
                 }
                 n2 = ldns_rbtree_next(n2);
@@ -342,22 +325,20 @@ zonelist_merge(zonelist_type* zl1, zonelist_type* zl2)
                 n2 = ldns_rbtree_next(n2);
                 zone_merge(z1, z2);
                 zone_cleanup(z2);
-                if (z1->just_updated) {
+                if (z1->zl_status == ZONE_ZL_UPDATED) {
                     zl1->just_updated++;
                 }
-                z1->just_updated = 1;
+                z1->zl_status = ZONE_ZL_UPDATED;
             }
         }
     }
-
     /* remove remaining zones from z1 */
     while (n1 && n1 != LDNS_RBTREE_NULL) {
         z1 = (zone_type*) n1->data;
-        z1->tobe_removed = 1;
+        z1->zl_status = ZONE_ZL_REMOVED;
         zl1->just_removed++;
         n1 = ldns_rbtree_next(n1);
     }
-
     zl1->last_modified = zl2->last_modified;
     return;
 }
@@ -375,46 +356,32 @@ zonelist_update(zonelist_type* zl, const char* zlfile)
     time_t st_mtime = 0;
     ods_status status = ODS_STATUS_OK;
     char* datestamp = NULL;
-    uint32_t ustamp = 0;
 
     ods_log_debug("[%s] update zone list", zl_str);
-    if (!zl|| !zl->zones) {
-        ods_log_error("[%s] cannot update: no zonelist storaga", zl_str);
+    if (!zl|| !zl->zones || !zlfile) {
         return ODS_STATUS_ASSERT_ERR;
     }
-    ods_log_assert(zl);
-    ods_log_assert(zl->zones);
-    if (!zlfile) {
-        ods_log_error("[%s] cannot update: no filename", zl_str);
-        return ODS_STATUS_ASSERT_ERR;
-    }
-    ods_log_assert(zlfile);
-
     /* is the file updated? */
     st_mtime = ods_file_lastmodified(zlfile);
     if (st_mtime <= zl->last_modified) {
-        ustamp = time_datestamp(zl->last_modified,
-            "%Y-%m-%d %T", &datestamp);
+        (void)time_datestamp(zl->last_modified, "%Y-%m-%d %T", &datestamp);
         ods_log_debug("[%s] zonelist file %s is unchanged since %s",
             zl_str, zlfile, datestamp?datestamp:"Unknown");
         free((void*)datestamp);
         return ODS_STATUS_UNCHANGED;
     }
-
     /* create new zonelist */
     tmp_alloc = allocator_create(malloc, free);
     if (!tmp_alloc) {
-        ods_log_error("[%s] error creating allocator for zone list",
-            zl_str);
-        return ODS_STATUS_ERR;
+        return ODS_STATUS_MALLOC_ERR;
     }
     new_zlist = zonelist_create(tmp_alloc);
     if (!new_zlist) {
-        ods_log_error("[%s] error creating new zone list", zl_str);
+        ods_log_error("[%s] unable to update zonelist: zonelist_create() "
+            "failed", zl_str);
         allocator_cleanup(tmp_alloc);
         return ODS_STATUS_ERR;
     }
-
     /* read zonelist */
     status = zonelist_read(new_zlist, zlfile);
     if (status == ODS_STATUS_OK) {
@@ -423,16 +390,14 @@ zonelist_update(zonelist_type* zl, const char* zlfile)
         zl->just_updated = 0;
         new_zlist->last_modified = st_mtime;
         zonelist_merge(zl, new_zlist);
-        ustamp = time_datestamp(zl->last_modified, "%Y-%m-%d %T",
-            &datestamp);
+        (void)time_datestamp(zl->last_modified, "%Y-%m-%d %T", &datestamp);
         ods_log_debug("[%s] file %s is modified since %s", zl_str, zlfile,
             datestamp?datestamp:"Unknown");
         free((void*)datestamp);
     } else {
-        ods_log_error("[%s] unable to read file %s: %s", zl_str, zlfile,
-            ods_status2str(status));
+        ods_log_error("[%s] unable to update zonelist: read file %s failed "
+            "(%s)", zl_str, zlfile, ods_status2str(status));
     }
-
     zonelist_free(new_zlist);
     allocator_cleanup(tmp_alloc);
     return status;
@@ -447,13 +412,11 @@ static void
 zone_delfunc(ldns_rbnode_t* elem)
 {
     zone_type* zone;
-
     if (elem && elem != LDNS_RBTREE_NULL) {
         zone = (zone_type*) elem->data;
         zone_delfunc(elem->left);
         zone_delfunc(elem->right);
-
-        ods_log_debug("[%s] cleanup zone %s", zl_str, zone->name);
+        ods_log_deeebug("[%s] cleanup zone %s", zl_str, zone->name);
         zone_cleanup(zone);
         free((void*)elem);
     }
@@ -486,21 +449,17 @@ zonelist_cleanup(zonelist_type* zl)
 {
     allocator_type* allocator;
     lock_basic_type zl_lock;
-
     if (!zl) {
         return;
     }
-
     ods_log_debug("[%s] cleanup zonelist", zl_str);
     if (zl->zones) {
         zone_delfunc(zl->zones->root);
         ldns_rbtree_free(zl->zones);
         zl->zones = NULL;
     }
-
     allocator = zl->allocator;
     zl_lock = zl->zl_lock;
-
     allocator_deallocate(allocator, (void*) zl);
     lock_basic_destroy(&zl_lock);
     return;
@@ -516,20 +475,16 @@ zonelist_free(zonelist_type* zl)
 {
     allocator_type* allocator;
     lock_basic_type zl_lock;
-
     if (!zl) {
         return;
     }
-
     if (zl->zones) {
         node_delfunc(zl->zones->root);
         ldns_rbtree_free(zl->zones);
         zl->zones = NULL;
     }
-
     allocator = zl->allocator;
     zl_lock = zl->zl_lock;
-
     allocator_deallocate(allocator, (void*) zl);
     lock_basic_destroy(&zl_lock);
     return;
